import ssl
import smtplib

from typing import List
from urllib.parse import urlparse

import httpx

from argos.checks.base import Severity
from argos.logging import logger
from argos.schemas.config import Config, Mail, GotifyUrl

# XXX Implement mail alerts https://framagit.org/framasoft/framaspace/argos/-/issues/15
# XXX Implement gotify alerts https://framagit.org/framasoft/framaspace/argos/-/issues/16

def handle_alert(config: Config, result, task, severity, request):
    """Dispatch alert through configured alert channels"""
    msg = f"task={task.id}, status={result.status}, {severity=}"
    logger.error("Alerting stub: %s", msg)

    if config.general.mail is not None or \
        config.general.gotify is not None:

        subject = f"{urlparse(task.url).netloc}: status {severity}"
        msg = f"""\
URL:    {task.url}
Check:  {task.check}
Status: {severity}
Time:   {result.submitted_at}

See results of task on {request.url_for('get_task_results', task_id=task.id)}
"""

        if config.general.mail is not None:
            notify_by_mail(subject, msg, config.general.mail)
        if config.general.gotify is not None:
            notify_with_gotify(subject, msg, severity, config.general.gotify)


def notify_by_mail(subject: str, msg: str, config: Mail) -> None:
    logger.debug('Will send mail notification')
    mail = f"""\
Subject: [Argos] {subject}

{msg}"""

    if config.ssl:
        logger.debug('Mail notification: SSL')
        context = ssl.create_default_context()
        smtp = smtplib.SMTP_SSL(host=config.host,
                                port=config.port,
                                context=context)
    else:
        smtp = smtplib.SMTP(host=config.host,  # type: ignore
                            port=config.port)
        if config.starttls:
            logger.debug('Mail notification: STARTTLS')
            context = ssl.create_default_context()
            smtp.starttls(context=context)

    if config.auth is not None:
        logger.debug('Mail notification: authentification')
        smtp.login(config.auth.login,
                   config.auth.password)

    for address in config.addresses:
        logger.debug('Sending mail to %s', address)
        logger.debug(msg)
        smtp.sendmail(config.mailfrom, address, mail)


def notify_with_gotify(subject: str, msg: str, severity: str, config: List[GotifyUrl]) -> None:
    logger.debug('Will send gotify notification')
    headers = {'accept': 'application/json',
               'content-type': 'application/json'}

    priority = 9
    if severity == Severity.OK:
        priority = 1
    elif severity == Severity.WARNING:
        priority = 5

    payload = {'title': subject,
               'message': msg,
               'priority': priority}

    for url in config:
        logger.debug('Sending gotify message(s) to %s', url)
        for token in url.tokens:
            try:
                res = httpx.post(f"{url.url}message",
                                 params={'token': token},
                                 headers=headers,
                                 json=payload)
                res.raise_for_status()
            except httpx.RequestError as exc:
                logger.error('An error occurred while sending a message to %s with token %s',
                             exc.request.url,
                             token)
